<?php

namespace FiloBlu\GcpLogging\Logger\Handler;

use FiloBlu\GcpLogging\Helper\Data;
use Google\Cloud\Logging\Logger;
use Google\Cloud\Logging\LoggingClient;
use Magento\Framework\Logger\Handler\Base as BaseHandler;

class Gcp extends BaseHandler
{
    /**
     * @var LoggerInterface[]
     */
    protected $loggers;

    /**
     * @var Data
     */
    protected $data;

    /**
     * Handler constructor.
     *
     * @param Data $data
     * @param bool|int $level
     * @param bool $bubble
     */
    public function __construct(Data $data, $level = Logger::DEBUG, $bubble = true)
    {
        $this->data = $data;
        $this->level = $level;
        $this->bubble = $bubble;
    }

    /**
     * {@inheritdoc}
     */
    public function handle(array $record)
    {
        if (!$this->data->gcpLoggingIsEnabled()) {
            return false;
        }

        if (!$this->isHandling($record)) {
            return false;
        }

        try {
            $this->getLogger($record['channel'])->log(strtolower($record['level_name']), $record['message'], $record['context']);

            return false === $this->bubble;
        } catch (\Exception $e) {
            return false;
        }
    }

    /**
     * @param $channel
     *
     * @return LoggerInterface
     */
    protected function getLogger($channel)
    {
        $client = new LoggingClient($this->data->getLoggingClientConfiguration());

        if (!isset($this->loggers[$channel])) {
            $this->loggers[$channel] = $client->psrLogger($this->data->getLogFileName(), ['labels' => ['context' => $channel]]);
        }

        return $this->loggers[$channel];
    }
}
